unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, JvComponentBase, JvHidControllerClass, HID, HidUsage, StdCtrls,
  ExtCtrls, ComCtrls, JvExComCtrls, JvComCtrls;

type
  TIRData = packed record
    Protocol : Byte;
    Address : Word;
    Command : Word;
    Flags : Byte;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportIRCode = packed record
    ReportID: Byte;
    Data: TIRData;
  end;

  // kein HID-Report kann groesser als 64 Byte sein
  THidIrmpVersion = packed record
    ReportID: Byte;
    Data: Array[0..10] of AnsiChar;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportIRPollingTime = packed record
    ReportID: Byte;
    Data: Word;
  end;

type
  // kein HID-Report kann groesser als 64 Byte sein
  THidReportPowerOn = packed record
    ReportID: Byte;
    Data: Boolean;
  end;

type
  THidReportMinRepeats = packed record
    ReportID: Byte;
    Data: Byte;
  end;

type
  TMainForm = class(TForm)
    JvHidDeviceController: TJvHidDeviceController;
    Memo1: TMemo;
    Panel1: TPanel;
    btSetIRCode: TButton;
    btPowerOnEnable: TButton;
    btPowerOnDisable: TButton;
    btReadTrainedIRCode: TButton;
    btPowerOnEnabled: TButton;
    btReadIrmpVersion: TButton;
    TrackBar_IR_Polling: TJvTrackBar;
    Label4: TLabel;
    Label2: TLabel;
    Label1: TLabel;
    Label3: TLabel;
    TrackBar_MinRepeats: TJvTrackBar;
    procedure OnArrival(HidDev: TJvHidDevice);
    procedure OnRemoval(HidDev: TJvHidDevice);
    procedure OnDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
      const Data: Pointer; Size: Word);
    procedure btSetIRCodeClick(Sender: TObject);
    procedure btPowerOnEnableClick(Sender: TObject);
    procedure btPowerOnDisableClick(Sender: TObject);
    procedure btReadTrainedIRCodeClick(Sender: TObject);
    procedure btPowerOnEnabledClick(Sender: TObject);
    procedure btReadIrmpVersionClick(Sender: TObject);
    procedure OnChange_IR_Pooling(Sender: TObject);
    procedure OnChanged_IR_Pooling(Sender: TObject);
    procedure OnChange_MinRepeats(Sender: TObject);
    procedure OnChanged_MinRepeats(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

const
  VendorID = $16C0;
  ProductID = $05DF;
  ProductName = 'USB IR Remote Receiver';

//irmp const:
const
  IRMP_SIRCS_PROTOCOL       = 1;
  IRMP_NEC_PROTOCOL         = 2;
  IRMP_SAMSUNG_PROTOCOL     = 3;
  IRMP_MATSUSHITA_PROTOCOL  = 4;
  IRMP_KASEIKYO_PROTOCOL    = 5;
  IRMP_RECS80_PROTOCOL      = 6;
  IRMP_RC5_PROTOCOL         = 7;
  IRMP_DENON_PROTOCOL       = 8;
  IRMP_RC6_PROTOCOL         = 9;
  IRMP_SAMSUNG32_PROTOCOL   = 10;
  IRMP_APPLE_PROTOCOL       = 11;
  IRMP_RECS80EXT_PROTOCOL   = 12;
  IRMP_NUBERT_PROTOCOL      = 13;


const
  NewIRCodeAvailable = 1;
  ReadPowerOnEnabled = 2;
  ReadTrainendIRCode = 3;
  SetPowerOnEnabled = 4;
  SetTrainedIRCode = 5;
  SetIRPollingTime = 6;
  ReadIrmpVersion = 7;
  SetMinRepeats = 8;


var
  MainForm: TMainForm;
  HIDDevice: TJvHidDevice;
  ReceivedData : TIRData;

  IRReport: THidReportIRCode;
  PowerOnReport : THidReportPowerOn;
  ReportIRPollingTime : THidReportIRPollingTime;
  IrmpVersion : THidIrmpVersion;
  MinRepeats : THidReportMinRepeats;

implementation

{$R *.dfm}
function GetIRProtocol(Index: Integer): String;
begin
  case Index of
    0 : Result := 'Unkown';
    IRMP_SIRCS_PROTOCOL : Result := 'SIRCS';
    IRMP_NEC_PROTOCOL : Result := 'NEC';
    IRMP_SAMSUNG_PROTOCOL : Result := 'SAMSUNG';
    IRMP_MATSUSHITA_PROTOCOL : Result := 'MATSUSHITA';
    IRMP_KASEIKYO_PROTOCOL : Result := 'KASEIKYO';
    IRMP_RECS80_PROTOCOL : Result := 'RECS80';
    IRMP_RC5_PROTOCOL : Result := 'RC5';
    IRMP_DENON_PROTOCOL : Result := 'DENON';
    IRMP_RC6_PROTOCOL : Result := 'RC6';
    IRMP_SAMSUNG32_PROTOCOL : Result := 'SAMSUNG32';
    IRMP_APPLE_PROTOCOL : Result := 'APPLE';
    IRMP_RECS80EXT_PROTOCOL : Result := 'RECS80EXT';
    IRMP_NUBERT_PROTOCOL : Result := 'NUBERT';
    else
      Result := 'Undefined';
  end;
end;

procedure TMainForm.OnDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
  const Data: Pointer; Size: Word);
begin
  if ReportID = 1 then
  begin
  CopyMemory(@ReceivedData, Data, Size);
  Memo1.Lines.Add(
    'Protocol : ' + GetIRProtocol(ReceivedData.Protocol)
    + ', Address: 0x' + IntToHex(ReceivedData.Address,4)
    + ', Command: 0x' + IntToHex(ReceivedData.Command,4)
    + ', Flags: 0x' + IntToHex(ReceivedData.Flags,2)
    );
  end;
end;

procedure TMainForm.btPowerOnDisableClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(PowerOnReport, SizeOf(PowerOnReport), 0);
  PowerOnReport.ReportID := SetPowerOnEnabled;
  PowerOnReport.Data := False;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(PowerOnReport, SizeOf(PowerOnReport));
end;

procedure TMainForm.btPowerOnEnableClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(PowerOnReport, SizeOf(PowerOnReport), 0);
  PowerOnReport.ReportID := SetPowerOnEnabled;
  PowerOnReport.Data := True;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(PowerOnReport, SizeOf(PowerOnReport));
end;

procedure TMainForm.btPowerOnEnabledClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(PowerOnReport, SizeOf(PowerOnReport), 0);
  PowerOnReport.ReportID := ReadPowerOnEnabled;

  if Assigned(HIDDevice) then
    if HIDDevice.GetFeature(PowerOnReport, SizeOf(PowerOnReport)) then
      Memo1.Lines.Add(
    'PowerOn Function status: '
    + IntToStr(Integer(PowerOnReport.Data))
    );
end;

procedure TMainForm.btReadIrmpVersionClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(IrmpVersion, SizeOf(IrmpVersion), 0);
  IrmpVersion.ReportID := ReadIrmpVersion;

  if Assigned(HIDDevice) then
    if HIDDevice.GetFeature(IrmpVersion, SizeOf(IrmpVersion)) then
      Memo1.Lines.Add(
    'Irmp Version: '
    + PAnsiChar(StrPas(IrmpVersion.Data))
    );
end;

procedure TMainForm.btReadTrainedIRCodeClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(IRReport, SizeOf(IRReport), 0);
  IRReport.ReportID := ReadTrainendIRCode;

  if Assigned(HIDDevice) then
    if HIDDevice.GetFeature(IRReport, SizeOf(IRReport)) then
      Memo1.Lines.Add(
    'Trained IR Code: '
    + 'Protocol : ' + GetIRProtocol(IRReport.Data.Protocol)
    + ', Address: 0x' + IntToHex(IRReport.Data.Address,4)
    + ', Command: 0x' + IntToHex(IRReport.Data.Command,4)
    + ', Flags: 0x' + IntToHex(IRReport.Data.Flags,2)
    );
end;

procedure TMainForm.btSetIRCodeClick(Sender: TObject);
begin
  //clear Reports:
  FillChar(IRReport, SizeOf(IRReport), 0);
  IRReport.ReportID := SetTrainedIRCode;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(IRReport, SizeOf(IRReport));
end;

procedure TMainForm.OnChange_IR_Pooling(Sender: TObject);
begin
  Label2.Caption := IntToStr(TrackBar_IR_Polling.Position) + 's';
end;

procedure TMainForm.OnChanged_MinRepeats(Sender: TObject);
begin
  //clear Reports:
  FillChar(MinRepeats, SizeOf(MinRepeats), 0);
  MinRepeats.ReportID := SetMinRepeats;
  MinRepeats.Data := TrackBar_MinRepeats.Position;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(MinRepeats, SizeOf(MinRepeats));
end;

procedure TMainForm.OnChange_MinRepeats(Sender: TObject);
begin
  Label3.Caption := IntToStr(TrackBar_MinRepeats.Position);
end;

procedure TMainForm.OnChanged_IR_Pooling(Sender: TObject);
var
  newFrequency : Extended;
  OCR1A : Word;
begin
  newFrequency := 1 / (TrackBar_IR_Polling.Position / 1000000);
  OCR1A := Trunc(newFrequency);

  //clear Reports:
  FillChar(ReportIRPollingTime, SizeOf(ReportIRPollingTime), 0);
  ReportIRPollingTime.ReportID := SetIRPollingTime;
  ReportIRPollingTime.Data := OCR1A;

  if Assigned(HIDDevice) then
    HIDDevice.SetFeature(ReportIRPollingTime, SizeOf(ReportIRPollingTime));
end;

procedure TMainForm.OnArrival(HidDev: TJvHidDevice);
var
  i : integer;
begin
  if ((HidDev.Attributes.VendorID = VendorID) AND
    (HidDev.Attributes.ProductID = ProductID) AND
    (HidDev.ProductName = ProductName )) then
  begin

    //store device:
    Memo1.Lines.Add('USB IR Remote Receiver connected');
    HIDDevice := HidDev;
    //device found, lets do the init:
    if HIDDevice.CheckOut then
      begin
        //set feature type
        HIDDevice.ReportTypeParam := HidP_Feature;
        HIDDevice.UsagePageParam := HID_USAGE_PAGE_GENERIC;
        HIDDevice.LinkCollectionParam := $01; // CollectionType=Application
        HIDDevice.UsageParam := HID_USAGE_CONSUMER_UNDEFINED;
      end;
  end;
end;

procedure TMainForm.OnRemoval(HidDev: TJvHidDevice);
begin
  if ((Assigned(HIDDevice)) and (NOT HIDDevice.IsPluggedIn)) then
  begin
    JvHidDeviceController.CheckIn(HIDDevice);
    Memo1.Lines.Add('USB IR Remote Receiver disconnected');
  end;
end;

end.
